#!/usr/bin/env python

"""
pdb2fasta.py

Receives pdb file from processPDB.py and sends back
a single string FASTA-like amino acid sequence format
which is accepted by checkHomology.py(function)
It tries to read SEQRES entries; failing which extracts sequence from
PDB coordinates listing.
"""

import os

class PdbSeqError(Exception):
    """
    Error class for this module.
    """

    pass


def getPdbSeq(pdb,use_atoms):
    """
    Parse the SEQRES entries in a pdb file.  If this fails, use the ATOM 
    entries. Returns dictionary of sequences keyed to chain and type of
    sequence used.
    """

    #Extract from SEQRES
    seq = [l for l in pdb if l[0:6] == "SEQRES"]
    if len(seq) != 0 and not use_atoms:
        seq_type = "SEQRES"
        chain_dict = dict([(l[11],[]) for l in seq])
        for c in chain_dict.keys():
            chain_seq = [l[19:70].split() for l in seq if l[11] == c]
            for x in chain_seq:
                chain_dict[c].extend(x)
    # Otherwise, use ATOM
    else:

        # Check to see if there are multiple models.  If there are, only look
        # at the first model.
        models = [i for i, l in enumerate(pdb) if l.startswith("MODEL")]
        if len(models) > 1:
            pdb = pdb[models[0]:models[1]]  

        seq_type = "ATOM  "
        atoms = [l for l in pdb if l[0:6] == "ATOM  " and l[13:16] == "CA "]
        chain_dict = dict([(l[21],[]) for l in atoms])
        for c in chain_dict.keys():
            chain_dict[c] = [l[17:20] for l in atoms if l[21] == c]

    return chain_dict, seq_type 


def pdb2Fasta(pdb,pdb_id,use_atoms,chain="all"):

    #ALL CHAINS TO BE CONSIDERED
    """
    AA dictionary for 3 letter code -> 1 letter code
    """
    _aa_index = [('ALA','A'),
                 ('CYS','C'),
                 ('ASP','D'),
                 ('GLU','E'),
                 ('PHE','F'),
                 ('GLY','G'),
                 ('HIS','H'),
                 ('ILE','I'),
                 ('LYS','K'),
                 ('LEU','L'),
                 ('MET','M'),
                 ('ASN','N'),
                 ('PRO','P'),
                 ('GLN','Q'),
                 ('ARG','R'),
                 ('SER','S'),
                 ('THR','T'),
                 ('VAL','V'),
                 ('TRP','W'),
                 ('TYR','Y')]

    AA3_TO_AA1 = dict(_aa_index)
    AA1_TO_AA3 = dict([(aa[1],aa[0]) for aa in _aa_index])

    """
    Extract sequence from pdb file and write out in FASTA format.
    """

    # Grab sequences
    chain_dict, seq_type = getPdbSeq(pdb,use_atoms)

    # Determine which chains are being written out
    if chain == "all":
        chains_to_write = chain_dict.keys()
        chains_to_write.sort()
    else:
        if chain in chain_dict.keys():
            chains_to_write = [chain]
        else:
            err = "Chain \"%s\" not in pdb!" % chain
            raise PdbSeqError(err)

    # Convert sequences to 1-letter format and join strings
    for c in chains_to_write:
        for aa_index, aa in enumerate(chain_dict[c]):
            try:
                chain_dict[c][aa_index] = AA3_TO_AA1[aa]
            except KeyError:
                chain_dict[c][aa_index] = "X"

    out = []
    for c in chains_to_write:
        seq_length = len(chain_dict[c])
              
        for i in range(seq_length):
            out.append("".join([aa for aa in chain_dict[c][i]]))
            
    return "".join(out)

def main():
    """
    Function to execute if called from command line.
    """

    import sys
    chain ="A"
    atomseq=False
    pdb_file="3SX3.pdb"
    
    pdb_id = os.path.split(pdb_file)[-1][:-4]
        
    f = open(pdb_file,'r')
    pdb = f.readlines()
    f.close()
        
    seq = pdb2Fasta(pdb,pdb_id,atomseq,chain)

    print type(seq),
    print seq
    


if __name__ == "__main__":
    main()
